@@ -876,7 +876,6 @@ typedef struct {
   _vid_playback_plugin *vpp;
 
   /// multi-head support
-  gint nmonitors;
   lives_mgeometry_t *mgeom;
 
 
@@ -925,6 +924,8 @@ typedef struct {
 
   GList *videodevs;
 
+  gchar vpp_defs_file[PATH_MAX];
+
 } mainwindow;
 
 GdkCursor *hidden_cursor;
