@@ -29,6 +29,8 @@
 #include "rfx-builder.h"
 #include "paramwindow.h"
 
+const char *anames[AUDIO_CODEC_MAX]={"mp3","pcm","mp2","vorbis","AC3","AAC","AMR_NB","raw",""};
+
 static gboolean list_plugins;
 
 
@@ -250,7 +252,7 @@ GList *get_plugin_list (const gchar *plugin_type, gboolean allow_nonex, const gc
 // video plugins
 
 
-void save_vpp_defaults(_vid_playback_plugin *vpp) {
+void save_vpp_defaults(_vid_playback_plugin *vpp, gchar *vpp_file) {
   // format is:
   // nbytes (string) : LiVES vpp defaults file version 2\n
   // for each video playback plugin:
@@ -274,7 +276,6 @@ void save_vpp_defaults(_vid_playback_plugin *vpp) {
   // n bytes string param value
 
   int fd;
-  gchar *vpp_file=g_strdup_printf("%s/%svpp_defaults",capable->home_dir,LIVES_CONFIG_DIR);
   gint32 len;
   const gchar *version;
   int i;
@@ -284,7 +285,6 @@ void save_vpp_defaults(_vid_playback_plugin *vpp) {
 
   if (mainw->vpp==NULL) {
     unlink(vpp_file);
-    g_free(vpp_file);
     return;
   }
 
@@ -292,7 +292,6 @@ void save_vpp_defaults(_vid_playback_plugin *vpp) {
     msg=g_strdup_printf (_("\n\nUnable to write video playback plugin defaults file\n%s\nError code %d\n"),vpp_file,errno);
     g_printerr ("%s",msg);
     g_free (msg);
-    g_free(vpp_file);
     return;
   }
 
@@ -332,14 +331,12 @@ void save_vpp_defaults(_vid_playback_plugin *vpp) {
   }
 
   close(fd);
-  g_free(vpp_file);
 
 }
 
 
-void load_vpp_defaults(_vid_playback_plugin *vpp) {
+void load_vpp_defaults(_vid_playback_plugin *vpp, gchar *vpp_file) {
   int fd;
-  gchar *vpp_file=g_strdup_printf("%s/%svpp_defaults",capable->home_dir,LIVES_CONFIG_DIR);
   gint32 len;
   const gchar *version;
   gchar buf[512];
@@ -347,7 +344,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
   gchar *msg;
 
   if (!g_file_test(vpp_file,G_FILE_TEST_EXISTS)) {
-    g_free(vpp_file);
     return;
   }
 
@@ -359,7 +355,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
     msg=g_strdup_printf (_("unable to read file\n%s\nError code %d\n"),vpp_file,errno);
     d_print (msg);
     g_free (msg);
-    g_free(vpp_file);
     return;
   }
 
@@ -371,7 +366,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
   if (strcmp(msg,buf)) {
     g_free(msg);
     d_print_file_error_failed();
-    g_free(vpp_file);
     return;
   }
   g_free(msg);
@@ -383,7 +377,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
 
   if (strcmp(buf,mainw->vpp->name)) {
     d_print_failed();
-    g_free(vpp_file);
     return;
   }
 
@@ -398,7 +391,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
     do_error_dialog(msg);
     g_free(msg);
     unlink(vpp_file);
-    g_free(vpp_file);
     d_print_failed();
     return;
   }
@@ -409,7 +401,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
   dummyvar=read(fd,&(mainw->vpp->YUV_subspace),sizint);
   dummyvar=read(fd,&(mainw->vpp->fwidth),sizint);
   dummyvar=read(fd,&(mainw->vpp->fheight),sizint);
-  //mainw->vpp->fwidth=mainw->vpp->fheight=-1;
   dummyvar=read(fd,&(mainw->vpp->fixed_fpsd),sizdbl);
   dummyvar=read(fd,&(mainw->vpp->fixed_fps_numer),sizint);
   dummyvar=read(fd,&(mainw->vpp->fixed_fps_denom),sizint);
@@ -435,8 +426,6 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
   mainw->vpp->extra_argv[i]=NULL;
 
   close(fd);
-  g_free(vpp_file);
-
   d_print_done();
 
 }
@@ -528,6 +517,7 @@ void on_vppa_ok_clicked (GtkButton *button, gpointer user_data) {
 
     if (mainw->vpp->fixed_fpsd>0.&&(mainw->fixed_fpsd>0.||!((*mainw->vpp->set_fps) (mainw->vpp->fixed_fpsd)))) {
       do_vpp_fps_error();
+      mainw->error=TRUE;
       mainw->vpp->fixed_fpsd=-1.;
       mainw->vpp->fixed_fps_numer=0;
     }
@@ -541,14 +531,22 @@ void on_vppa_ok_clicked (GtkButton *button, gpointer user_data) {
 	      if (mainw->vpp->exit_screen!=NULL) {
 		(*mainw->vpp->exit_screen)(mainw->ptr_x,mainw->ptr_y);
 	      }
+
+	      stop_audio_stream();
+
 	      mainw->stream_ticks=-1;
 	      mainw->vpp->palette=pal_list[i];
 	      if (!(*vpp->set_palette)(vpp->palette)) {
 		do_vpp_palette_error();
+		mainw->error=TRUE;
 	      }
 
 	      if (vpp->set_yuv_palette_clamping!=NULL) (*vpp->set_yuv_palette_clamping)(vpp->YUV_clamping);
 
+	      if (mainw->vpp->audio_codec!=AUDIO_CODEC_NONE&&prefs->stream_audio_out) {
+		start_audio_stream();
+	      }
+
 	      if (vpp->init_screen!=NULL) {
 		(*vpp->init_screen)(mainw->pwidth,mainw->pheight,TRUE,0,vpp->extra_argc,vpp->extra_argv);
 	      }
@@ -557,6 +555,7 @@ void on_vppa_ok_clicked (GtkButton *button, gpointer user_data) {
 	      mainw->vpp->palette=pal_list[i];
 	      if (!(*vpp->set_palette)(vpp->palette)) {
 		do_vpp_palette_error();
+		mainw->error=TRUE;
 	      }
 	      if (vpp->set_yuv_palette_clamping!=NULL) (*vpp->set_yuv_palette_clamping)(vpp->YUV_clamping);
 	    }
@@ -656,11 +655,38 @@ void on_vppa_ok_clicked (GtkButton *button, gpointer user_data) {
       vpp->extra_argc=0;
     }
   }
-  on_vppa_cancel_clicked(button,user_data);
+  if (button!=NULL&&!mainw->error) on_vppa_cancel_clicked(button,user_data);
+  if (button!=NULL) mainw->error=FALSE;
 }
 
 
+void on_vppa_save_clicked (GtkButton *button, gpointer user_data) {
+  _vppaw *vppw=(_vppaw *)user_data;
+  _vid_playback_plugin *vpp=vppw->plugin;
+  gchar *save_file;
+  gchar *msg;
 
+  // apply
+  mainw->error=FALSE;
+  on_vppa_ok_clicked(NULL,user_data);
+  if (mainw->error) {
+    mainw->error=FALSE;
+    return;
+  }
+
+  // get filename
+  save_file=choose_file(NULL,NULL,NULL,GTK_FILE_CHOOSER_ACTION_SAVE,NULL);
+  if (save_file==NULL) return;
+
+  // save
+  msg=g_strdup_printf(_("Saving playback plugin defaults to %s..."),save_file);
+  d_print(msg);
+  g_free(msg);
+  save_vpp_defaults(vpp, save_file);
+  d_print_done();
+  g_free(save_file);
+
+}
 
 
 _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
@@ -670,6 +696,7 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
   GtkWidget *combo;
   GtkWidget *cancelbutton;
   GtkWidget *okbutton;
+  GtkWidget *savebutton;
   GtkObject *spinbutton_adj;
 
   gchar *title;
@@ -709,12 +736,19 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
 
   vppa->dialog = gtk_dialog_new ();
 
-  //if (prefsw!=NULL) gtk_widget_hide (prefsw->prefs_dialog);
-
   gtk_window_set_position (GTK_WINDOW (vppa->dialog), GTK_WIN_POS_CENTER);
-  if (!prefs->show_gui) {
-    gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(mainw->LiVES));
+  if (prefs->show_gui) {
+    if (prefsw!=NULL) gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(prefsw->prefs_dialog));
+    else {
+      if (mainw->multitrack==NULL) gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(mainw->LiVES));
+      else gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(mainw->multitrack->window));
+    }
   }
+
+  if (prefs->gui_monitor!=0) {
+    gtk_window_set_screen(GTK_WINDOW(vppa->dialog),mainw->mgeom[prefs->gui_monitor-1].screen);
+  }
+
   gtk_window_set_modal (GTK_WINDOW (vppa->dialog), TRUE);
 
   if (palette->style&STYLE_1) {
@@ -776,7 +810,6 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
     vppa->fps_entry=(GTK_COMBO(combo))->entry;
 
     gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO (combo))->entry),FALSE);
-    //gtk_entry_set_activates_default(GTK_ENTRY((GTK_COMBO(combo))->entry),TRUE);
     
     nfps=get_token_count((gchar *)fps_list,'|');
     for (i=0;i<nfps;i++) {
@@ -822,7 +855,7 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
     
     spinbutton_adj = gtk_adjustment_new (tmpvpp->fwidth>0?tmpvpp->fwidth:DEF_VPP_HSIZE, 4., 100000, 4, 4, 0.);
     vppa->spinbuttonw = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 4, 0);
-    //gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbuttonw)->entry)), TRUE);
+
     gtk_label_set_mnemonic_widget (GTK_LABEL (label),vppa->spinbuttonw);
     gtk_box_pack_start (GTK_BOX (hbox), vppa->spinbuttonw, FALSE, FALSE, 10);
     gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (vppa->spinbuttonw),GTK_UPDATE_IF_VALID);
@@ -837,7 +870,7 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
     
     spinbutton_adj = gtk_adjustment_new (tmpvpp->fheight>0?tmpvpp->fheight:DEF_VPP_VSIZE, 4., 100000, 4, 4, 0.);
     vppa->spinbuttonh = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 4, 0);
-    //gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbuttonw)->entry)), TRUE);
+
     gtk_label_set_mnemonic_widget (GTK_LABEL (label),vppa->spinbuttonh);
     gtk_box_pack_start (GTK_BOX (hbox), vppa->spinbuttonh, FALSE, FALSE, 10);
     gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (vppa->spinbuttonh),GTK_UPDATE_IF_VALID);
@@ -923,7 +956,12 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
   cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
   gtk_widget_show (cancelbutton);
   gtk_dialog_add_action_widget (GTK_DIALOG (vppa->dialog), cancelbutton, GTK_RESPONSE_CANCEL);
-  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);
+
+  savebutton = gtk_button_new_from_stock ("gtk-save-as");
+  gtk_widget_show (savebutton);
+  gtk_dialog_add_action_widget (GTK_DIALOG (vppa->dialog), savebutton, 1);
+  GTK_WIDGET_SET_FLAGS (savebutton, GTK_CAN_DEFAULT);
+  gtk_tooltips_set_tip (mainw->tooltips, savebutton, _("Save settings to an alternate file.\n"), NULL);
 
   okbutton = gtk_button_new_from_stock ("gtk-ok");
   gtk_widget_show (okbutton);
@@ -934,6 +972,9 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
 		    G_CALLBACK (on_vppa_cancel_clicked),
 		    vppa);
 
+  g_signal_connect (GTK_OBJECT (savebutton), "clicked",
+		    G_CALLBACK (on_vppa_save_clicked),
+		    vppa);
 
   g_signal_connect (GTK_OBJECT (okbutton), "clicked",
 		    G_CALLBACK (on_vppa_ok_clicked),
@@ -954,6 +995,7 @@ void close_vid_playback_plugin(_vid_playback_plugin *vpp) {
   if (vpp!=NULL) {
     if (vpp==mainw->vpp) {
       if (mainw->ext_playback&&mainw->vpp->exit_screen!=NULL) (*mainw->vpp->exit_screen)(mainw->ptr_x,mainw->ptr_y);
+      stop_audio_stream();
       mainw->stream_ticks=-1;
       mainw->vpp=NULL;
     }
@@ -1053,11 +1095,11 @@ _vid_playback_plugin *open_vid_playback_plugin (const gchar *name, gboolean usin
   vpp->get_yuv_palette_clamping=dlsym (handle,"get_yuv_palette_clamping");
   vpp->set_yuv_palette_clamping=dlsym (handle,"set_yuv_palette_clamping");
   vpp->send_keycodes=dlsym (handle,"send_keycodes");
+  vpp->get_audio_fmts=dlsym (handle,"get_audio_fmts");
   vpp->init_screen=dlsym (handle,"init_screen");
   vpp->exit_screen=dlsym (handle,"exit_screen");
   vpp->module_unload=dlsym (handle,"module_unload");
 
-
   vpp->YUV_sampling=0;
   vpp->YUV_subspace=0;
 
@@ -1078,6 +1120,7 @@ _vid_playback_plugin *open_vid_playback_plugin (const gchar *name, gboolean usin
     }
   }
 
+  vpp->audio_codec=AUDIO_CODEC_NONE;
   vpp->capabilities=(*vpp->get_capabilities)(vpp->palette);
 
   if (vpp->capabilities&VPP_CAN_RESIZE) {
@@ -1161,6 +1204,10 @@ _vid_playback_plugin *open_vid_playback_plugin (const gchar *name, gboolean usin
     }
   }
 
+  if (vpp->get_audio_fmts!=NULL&&mainw->is_ready) vpp->audio_codec=get_best_audio(vpp);
+
+  if (prefsw!=NULL) prefsw_set_astream_settings(vpp);
+
   if (!using) return vpp;
 
   if (!mainw->is_ready) {
@@ -1169,7 +1216,7 @@ _vid_playback_plugin *open_vid_playback_plugin (const gchar *name, gboolean usin
     gint fheight=vpp->fheight;
 
     mainw->vpp=vpp;
-    load_vpp_defaults(vpp);
+    load_vpp_defaults(vpp, mainw->vpp_defs_file);
     if (fixed_fpsd<0.) vpp->fixed_fpsd=fixed_fpsd;
     if (fwidth<0) vpp->fwidth=fwidth;
     if (fheight<0) vpp->fheight=fheight;
@@ -1222,6 +1269,7 @@ void vid_playback_plugin_exit (void) {
   // external plugin
   if (mainw->ext_playback) {
     if (mainw->vpp->exit_screen!=NULL) (*mainw->vpp->exit_screen)(mainw->ptr_x,mainw->ptr_y);
+    stop_audio_stream();
     mainw->ext_playback=FALSE;
   }
   mainw->stream_ticks=-1;
@@ -1231,6 +1279,62 @@ void vid_playback_plugin_exit (void) {
   gtk_window_set_title (GTK_WINDOW (mainw->play_window),_("LiVES: - Play Window"));
 }
 
+
+gint64 get_best_audio(_vid_playback_plugin *vpp) {
+  // find best audio from video plugin list, matching with audiostream plugins
+  int *fmts,*sfmts;
+  int ret=AUDIO_CODEC_NONE;
+  int i,j=0,nfmts;
+  size_t rlen;
+  gchar *astreamer,*com,*playername;
+  gchar buf[1024];
+  gchar **array;
+  FILE *rfile;
+
+  if (vpp!=NULL&&vpp->get_audio_fmts!=NULL) {
+    fmts=(*vpp->get_audio_fmts)();
+
+    // make audiostream plugin name
+    playername=g_strdup_printf("%sstreamer.pl",prefs->aplayer);
+    astreamer=g_build_filename(prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_AUDIO_STREAM,playername,NULL);
+    g_free(playername);
+
+    // create sfmts array and nfmts
+
+    com=g_strdup_printf("%s get_formats",astreamer);
+    g_free(astreamer);
+
+    rfile=popen(com,"r");
+    rlen=fread(buf,1,1023,rfile);
+    pclose(rfile);
+    memset(buf+rlen,0,1);
+    g_free(com);
+
+    nfmts=get_token_count(buf,'|');
+    array=g_strsplit(buf,"|",nfmts);
+    sfmts=g_malloc(nfmts*sizint);
+
+    for (i=0;i<nfmts;i++) {
+      if (array[i]!=NULL&&strlen(array[i])>0) sfmts[j++]=atoi(array[i]);
+    }
+
+    nfmts=j;
+    g_strfreev(array);
+
+    for (i=0;fmts[i]!=-1;i++) {
+      // traverse video list and see if audiostreamer supports each one
+      if (int_array_contains_value(sfmts,nfmts,fmts[i])) {
+	ret=fmts[i];
+	break;
+      }
+    }
+    g_free(sfmts);
+  }
+
+  return ret;
+}
+
+
 ///////////////////////
 // encoder plugins
 
